#!/usr/bin/env bash

set -eu

util_path="${GITHUB_ACTION_PATH:-.}/scripts/utils.sh"

# shellcheck source=scripts/utils.sh
. "${util_path}"

group 'Retrieving Flake information'
flake_info=$(nix flake show --json)
endgroup

group 'Show Packages'
packages=$(
  jq --raw-output '.packages."x86_64-linux" | select(. != null) | keys[]' <<-EOF
		$flake_info
	EOF
)
log "$packages"
endgroup

group 'Show Home Manager Configurations'
hmConfigs=$(
  jq --raw-output '.homeConfigurations | select(. != null) | keys[]' <<-EOF
		$flake_info
	EOF
)
log "$hmConfigs"
endgroup

group 'Show NixOS Configurations'
nixosConfigs=$(
  jq --raw-output '.nixosConfigurations | select(. != null) | keys[]' <<-EOF
		$flake_info
	EOF
)
log "$nixosConfigs"
endgroup

pkgs_json() {
  group 'Building packages'
  trap endgroup RETURN
  for package in $packages; do
    log "Building $package"
    path=$(nix build --print-out-paths ".#$package")
    log "Calculating size of $package"
    nix path-info --closure-size --json "$path" |
      jq --compact-output --arg pkg "$package" '.[] | {"name": $pkg, "size": .closureSize, "narSize": .narSize}'
  done
  endgroup
}

hm_configs_json() {
  group 'Building Home Manager configurations'
  trap endgroup RETURN
  for config in $hmConfigs; do
    log "Building $config"
    path=$(nix build --print-out-paths ".#homeConfigurations.$config.activationPackages")
    log "Calculating size of $config"
    nix path-info --closure-size --json "$path" |
      jq --compact-output --arg pkg "$config" '.[] | {"name": $pkg, "size": .closureSize, "narSize": .narSize}'
  done
}

nixos_configs_json() {
  group 'Building NixOS configurations'
  trap endgroup RETURN
  for config in $nixosConfigs; do
    log "Building $config"
    path=$(nix build --print-out-paths ".#nixosConfigurations.$config.config.system.build.toplevel")
    log "Calculating size of $config"
    nix path-info --closure-size --json "$path" |
      jq --compact-output --arg pkg "$config" '.[] | {"name": $pkg, "size": .closureSize, "narSize": .narSize}'
  done
}

pkgs=$(pkgs_json | jq --slurp '.')
hmConfigs=$(hm_configs_json | jq --slurp '.')
nixosConfigs=$(nixos_configs_json | jq --slurp '.')

echo "{}" | jq \
  --argjson pkgs "$pkgs" \
  --argjson nixosConfigs "$nixosConfigs" \
  --argjson hmConfigs "$hmConfigs" \
  '{"packages": $pkgs, "nixosConfigurations": $nixosConfigs, "homeConfigurations": $hmConfigs}'
